#ifndef OLAF_FP_DB_H
#define OLAF_FP_DB_H
	#include <stdbool.h>
	#include <stdint.h>

	// The 'database' is a serialized sorted array of uint64_t elements
	// Of the 64 bits:
	// - The 18 most significant bits are a hash
	// - The next 14 bits a time stamp
	// - The remaining 32 bits are an identifier of a song
	//
	// About 13k fingerprints are used for a 4min song
	// 13k * 64bit =  100kB, with 20GB memory, 250k songs can be stored
	//
	// There is no overhead for disk storage size
	typedef struct Olaf_FP_DB Olaf_FP_DB;

	//create a new database, if the file name exists, read the contents
	Olaf_FP_DB * olaf_fp_db_new(const char * db_file_folder,bool readonly);

	//store a list of elements in the memory store
	void olaf_fp_db_store(Olaf_FP_DB *, uint32_t * keys, uint64_t * values, size_t size);

	//find a list of elements in the memory store
	//only take into account the x most significant bits
	void olaf_fp_db_find(Olaf_FP_DB *,uint32_t key,int bits_to_ignore,uint64_t * results, size_t results_size, size_t * number_of_results);

	//Does the db contain this key?
	bool olaf_fp_db_find_single(Olaf_FP_DB *,uint32_t key,int bits_to_ignore);


	//free memory resources and close resources
	void olaf_fp_db_destroy(Olaf_FP_DB * );

	//print database statistics
	void olaf_fp_db_stats(Olaf_FP_DB * olaf_fp_db);

	//hash
	uint32_t olaf_fp_db_string_hash(const char *key, size_t len);

#endif // OLAF_FP_DB_H
